@@ -4,13 +4,14 @@ import (
 	"crypto/tls"
 	"crypto/x509"
 	"fmt"
+	"reflect"
+	"strings"
+	"time"
+
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/serializer"
-	"reflect"
-	"strings"
-	"time"
 
 	. "github.com/onsi/ginkgo"
 	"github.com/onsi/ginkgo/extensions/table"
@@ -33,7 +34,7 @@ import (
 	validator "kubevirt.io/ssp-operator/internal/operands/template-validator"
 )
 
-var _ = Describe("Template validator", func() {
+var _ = Describe("Template validator operand", func() {
 	var (
 		clusterRoleRes        testResource
 		clusterRoleBindingRes testResource
@@ -346,38 +347,45 @@ var _ = Describe("Template validator", func() {
 			Expect(err).ToNot(HaveOccurred(), "SSP status should be deployed.")
 		})
 	})
+})
 
-	Context("Validation rules tests", func() {
-		var (
-			vm       *kubevirtv1.VirtualMachine
-			vmi      *kubevirtv1.VirtualMachineInstance
-			template *templatev1.Template
-		)
-		const (
-			TemplateNameAnnotation      = "vm.kubevirt.io/template"
-			TemplateNamespaceAnnotation = "vm.kubevirt.io/template-namespace"
-		)
+var _ = Describe("Template validator webhooks", func() {
+	var (
+		vm       *kubevirtv1.VirtualMachine
+		vmi      *kubevirtv1.VirtualMachineInstance
+		template *templatev1.Template
+	)
+	const (
+		TemplateNameAnnotation      = "vm.kubevirt.io/template"
+		TemplateNamespaceAnnotation = "vm.kubevirt.io/template-namespace"
+	)
 
-		BeforeEach(func() {
-			vmi = NewRandomVMIWithBridgeInterface(strategy.GetNamespace())
-			vm = nil
-			template = nil
-		})
-		AfterEach(func() {
-			if template != nil {
-				err := apiClient.Delete(ctx, template)
-				if !errors.IsNotFound(err) {
-					Expect(err).ToNot(HaveOccurred(), "Failed to delete Template")
-				}
+	BeforeEach(func() {
+		vmi = NewRandomVMIWithBridgeInterface(strategy.GetNamespace())
+		vm = nil
+		template = nil
+
+		waitUntilDeployed()
+	})
+	AfterEach(func() {
+		if vm != nil {
+			err := apiClient.Delete(ctx, vm)
+			if !errors.IsNotFound(err) {
+				Expect(err).ToNot(HaveOccurred(), "Failed to Delete VM")
 			}
-			if vm != nil {
-				err := apiClient.Delete(ctx, vm)
-				if !errors.IsNotFound(err) {
-					Expect(err).ToNot(HaveOccurred(), "Failed to Delete VM")
-				}
+			// Need to wait until VM is removed, otherwise the webhook may
+			// not allow template to be removed.
+			waitForDeletion(client.ObjectKeyFromObject(vm), vm)
+		}
+		if template != nil {
+			err := apiClient.Delete(ctx, template)
+			if !errors.IsNotFound(err) {
+				Expect(err).ToNot(HaveOccurred(), "Failed to delete Template")
 			}
-		})
+		}
+	})
 
+	Context("Validation defined on template", func() {
 		It("[test_id:5584]should create VM without template", func() {
 			vm = NewVirtualMachine(vmi)
 			Expect(apiClient.Create(ctx, vm)).ToNot(HaveOccurred(), "Failed to create VM")
@@ -408,7 +416,6 @@ var _ = Describe("Template validator", func() {
 		It("[test_id:2960] Negative test - Create a VM with machine type violation", func() {
 			template = TemplateWithRules()
 			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
-
 			// set value unfulfilling validation
 			vmi = addDomainResourcesToVMI(vmi, 2, "test", "128M")
 			vm = NewVirtualMachine(vmi)
@@ -418,6 +425,22 @@ var _ = Describe("Template validator", func() {
 			}
 			eventuallyFailToCreateVm(vm)
 		})
+
+		It("[test_id:7060]vm rejected metrics increses by one when vm is rejected", func() {
+			rejectedCountBefore := totalRejectedVmsMetricsValue()
+			template = TemplateWithRules()
+			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
+			// set value unfulfilling validation
+			vmi = addDomainResourcesToVMI(vmi, 2, "test", "128M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+			}
+			eventuallyFailToCreateVm(vm)
+			Expect(totalRejectedVmsMetricsValue() - rejectedCountBefore).To(Equal(1))
+		})
+
 		It("[test_id:5586]test with template optional rules unfulfilled", func() {
 			template = TemplateWithRulesOptional()
 			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
@@ -543,17 +566,19 @@ var _ = Describe("Template validator", func() {
 			}
 			eventuallyCreateVm(vm)
 		})
-		Context("with Validation inside VM object", func() {
-			It("[test_id:5173]: should create a VM that passes validation", func() {
-				template = TemplateWithoutRules()
-				Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
+	})
 
-				vmi = addDomainResourcesToVMI(vmi, 1, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					TemplateNameAnnotation:      template.Name,
-					TemplateNamespaceAnnotation: template.Namespace,
-					"vm.kubevirt.io/validations": `[
+	Context("Validation inside VM object", func() {
+		It("[test_id:5173]: should create a VM that passes validation", func() {
+			template = TemplateWithoutRules()
+			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
+
+			vmi = addDomainResourcesToVMI(vmi, 1, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -563,19 +588,19 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyCreateVm(vm)
-			})
-			It("[test_id:5034]: should fail to create VM that fails validation", func() {
-				template = TemplateWithoutRules()
-				Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
+			}
+			eventuallyCreateVm(vm)
+		})
+		It("[test_id:5034]: should fail to create VM that fails validation", func() {
+			template = TemplateWithoutRules()
+			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
 
-				vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					TemplateNameAnnotation:      template.Name,
-					TemplateNamespaceAnnotation: template.Namespace,
-					"vm.kubevirt.io/validations": `[
+			vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -585,19 +610,19 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyFailToCreateVm(vm)
-			})
-			It("[test_id:5035]: Template with validations, VM with validations", func() {
-				template = TemplateWithRules()
-				Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
+			}
+			eventuallyFailToCreateVm(vm)
+		})
+		It("[test_id:5035]: Template with validations, VM with validations", func() {
+			template = TemplateWithRules()
+			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
 
-				vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					TemplateNameAnnotation:      template.Name,
-					TemplateNamespaceAnnotation: template.Namespace,
-					"vm.kubevirt.io/validations": `[
+			vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -607,19 +632,19 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyFailToCreateVm(vm)
-			})
-			It("[test_id:5036]: should successfully create a VM based on the VM validation rules priority over template rules", func() {
-				template = TemplateWithRules()
-				Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
+			}
+			eventuallyFailToCreateVm(vm)
+		})
+		It("[test_id:5036]: should successfully create a VM based on the VM validation rules priority over template rules", func() {
+			template = TemplateWithRules()
+			Expect(apiClient.Create(ctx, template)).ToNot(HaveOccurred(), "Failed to create template: %s", template.Name)
 
-				vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					TemplateNameAnnotation:      template.Name,
-					TemplateNamespaceAnnotation: template.Namespace,
-					"vm.kubevirt.io/validations": `[
+			vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -629,16 +654,16 @@ var _ = Describe("Template validator", func() {
 														"max": 6
         										 }
 												]`,
-				}
-				eventuallyCreateVm(vm)
-			})
-			It("[test_id:5174]: VM with validations and deleted template", func() {
-				vmi = addDomainResourcesToVMI(vmi, 3, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					TemplateNameAnnotation:      "nonexisting-vm-template",
-					TemplateNamespaceAnnotation: strategy.GetTemplatesNamespace(),
-					"vm.kubevirt.io/validations": `[
+			}
+			eventuallyCreateVm(vm)
+		})
+		It("[test_id:5174]: VM with validations and deleted template", func() {
+			vmi = addDomainResourcesToVMI(vmi, 3, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      "nonexisting-vm-template",
+				TemplateNamespaceAnnotation: strategy.GetTemplatesNamespace(),
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -648,16 +673,16 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyCreateVm(vm)
-			})
-			It("[test_id:5046]: should override template rules and fail to create a VM based on the VM validation rules", func() {
-				vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					TemplateNameAnnotation:      "nonexisting-vm-template",
-					TemplateNamespaceAnnotation: strategy.GetTemplatesNamespace(),
-					"vm.kubevirt.io/validations": `[
+			}
+			eventuallyCreateVm(vm)
+		})
+		It("[test_id:5046]: should override template rules and fail to create a VM based on the VM validation rules", func() {
+			vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      "nonexisting-vm-template",
+				TemplateNamespaceAnnotation: strategy.GetTemplatesNamespace(),
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -667,14 +692,14 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyFailToCreateVm(vm)
-			})
-			It("[test_id:5047]: should fail to create a VM based on the VM validation rules", func() {
-				vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					"vm.kubevirt.io/validations": `[
+			}
+			eventuallyFailToCreateVm(vm)
+		})
+		It("[test_id:5047]: should fail to create a VM based on the VM validation rules", func() {
+			vmi = addDomainResourcesToVMI(vmi, 5, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -684,14 +709,14 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyFailToCreateVm(vm)
-			})
-			It("[test_id:5175]: VM with validations without template", func() {
-				vmi = addDomainResourcesToVMI(vmi, 3, "q35", "64M")
-				vm = NewVirtualMachine(vmi)
-				vm.ObjectMeta.Annotations = map[string]string{
-					"vm.kubevirt.io/validations": `[
+			}
+			eventuallyFailToCreateVm(vm)
+		})
+		It("[test_id:5175]: VM with validations without template", func() {
+			vmi = addDomainResourcesToVMI(vmi, 3, "q35", "64M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				"vm.kubevirt.io/validations": `[
 												 {
 														"name": "LimitCores",
 														"path": "jsonpath::.spec.domain.cpu.cores",
@@ -701,41 +726,113 @@ var _ = Describe("Template validator", func() {
 														"max": 4
         										 }
 												]`,
-				}
-				eventuallyCreateVm(vm)
-			})
+			}
+			eventuallyCreateVm(vm)
 		})
 	})
 
-	Context("Certificates", func() {
-		It("[test_id:4375] Test refreshing of certificates", func() {
-			pods, err := GetRunningPodsByLabel(validator.VirtTemplateValidator, validator.KubevirtIo, strategy.GetNamespace())
-			Expect(err).ToNot(HaveOccurred())
-			Expect(pods.Items).ToNot(HaveLen(0), "no pods found")
+	Context("Deleting template", func() {
+		It("[test_id:7026] Can delete template if no VM uses it", func() {
+			template = TemplateWithRules()
+			Expect(apiClient.Create(ctx, template)).To(Succeed())
 
-			validatorPod := pods.Items[0]
-			oldCerts, err := getWebhookServerCertificates(&validatorPod)
-			Expect(err).ToNot(HaveOccurred())
-
-			By("deleting the secret with certificate")
-			secret := &core.Secret{
-				ObjectMeta: metav1.ObjectMeta{
-					Name:      validator.SecretName,
-					Namespace: strategy.GetNamespace(),
-				},
-			}
-			err = apiClient.Delete(ctx, secret)
-			Expect(err).ToNot(HaveOccurred())
-
-			By("checking that the secret gets restored with a new certificate")
-			Eventually(func() (bool, error) {
-				newCerts, err := getWebhookServerCertificates(&validatorPod)
-				if err != nil {
-					return true, err
-				}
-				return certsEqual(newCerts, oldCerts), nil
-			}, 5*time.Minute, 1*time.Second).Should(BeFalse(), "Certificates should be different")
+			Consistently(func() error {
+				return apiClient.Delete(ctx, template, client.DryRunAll)
+			}, 5*time.Second, 500*time.Millisecond).Should(Succeed())
 		})
+
+		It("[test_id:7027] Should fail to delete template if a VM uses it", func() {
+			template = TemplateWithRules()
+			Expect(apiClient.Create(ctx, template)).To(Succeed())
+
+			vmi = addDomainResourcesToVMI(vmi, 2, "q35", "128M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+			}
+			eventuallyCreateVm(vm)
+
+			Eventually(func() metav1.StatusReason {
+				err := apiClient.Delete(ctx, template, client.DryRunAll)
+				return errors.ReasonForError(err)
+			}, shortTimeout, 1*time.Second).Should(Equal(metav1.StatusReasonForbidden), "Should have given forbidden error")
+		})
+
+		It("[test_id:7037] Can delete template without validations if a VM uses it", func() {
+			template = TemplateWithoutRules()
+			delete(template.Annotations, "validations")
+
+			Expect(apiClient.Create(ctx, template)).To(Succeed())
+
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+			}
+			eventuallyCreateVm(vm)
+
+			Consistently(func() error {
+				return apiClient.Delete(ctx, template, client.DryRunAll)
+			}, 5*time.Second, 500*time.Millisecond).Should(Succeed())
+		})
+
+		It("[test_id:7035] Can delete template if validations are defined on VM", func() {
+			template = TemplateWithRules()
+			Expect(apiClient.Create(ctx, template)).To(Succeed())
+
+			vmi = addDomainResourcesToVMI(vmi, 2, "q35", "128M")
+			vm = NewVirtualMachine(vmi)
+			vm.ObjectMeta.Annotations = map[string]string{
+				TemplateNameAnnotation:      template.Name,
+				TemplateNamespaceAnnotation: template.Namespace,
+				"vm.kubevirt.io/validations": `[
+												 {
+														"name": "LimitCores",
+														"path": "jsonpath::.spec.domain.cpu.cores",
+														"message": "Core amount not within range",
+														"rule": "integer",
+														"min": 1,
+														"max": 4
+        										 }
+												]`,
+			}
+
+			eventuallyCreateVm(vm)
+
+			Consistently(func() error {
+				return apiClient.Delete(ctx, template, client.DryRunAll)
+			}, 5*time.Second, 500*time.Millisecond).Should(Succeed())
+		})
+	})
+
+	It("[test_id:4375] Test refreshing of certificates", func() {
+		pods, err := GetRunningPodsByLabel(validator.VirtTemplateValidator, validator.KubevirtIo, strategy.GetNamespace())
+		Expect(err).ToNot(HaveOccurred())
+		Expect(pods.Items).ToNot(HaveLen(0), "no pods found")
+
+		validatorPod := pods.Items[0]
+		oldCerts, err := getWebhookServerCertificates(&validatorPod)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("deleting the secret with certificate")
+		secret := &core.Secret{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      validator.SecretName,
+				Namespace: strategy.GetNamespace(),
+			},
+		}
+		err = apiClient.Delete(ctx, secret)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("checking that the secret gets restored with a new certificate")
+		Eventually(func() (bool, error) {
+			newCerts, err := getWebhookServerCertificates(&validatorPod)
+			if err != nil {
+				return true, err
+			}
+			return certsEqual(newCerts, oldCerts), nil
+		}, 5*time.Minute, 1*time.Second).Should(BeFalse(), "Certificates should be different")
 	})
 })
 
@@ -775,6 +872,16 @@ func eventuallyFailToCreateVm(vm *kubevirtv1.VirtualMachine) bool {
 	}, shortTimeout).Should(Equal(metav1.StatusReasonInvalid), "Should have given the invalid error")
 }
 
+func totalRejectedVmsMetricsValue() (sum int) {
+	pods, err := GetRunningPodsByLabel(validator.VirtTemplateValidator, validator.KubevirtIo, strategy.GetNamespace())
+	Expect(err).ToNot(HaveOccurred(), "Could not find the validator pods")
+	Expect(pods.Items).ToNot(BeEmpty())
+	for _, validatorPod := range pods.Items {
+		sum += intMetricValue("total_rejected_vms", validator.MetricsPort, &validatorPod)
+	}
+	return
+}
+
 func addObjectsToTemplates(genName, validation string) *templatev1.Template {
 	editable := `/objects[0].spec.template.spec.domain.cpu.sockets
 				/objects[0].spec.template.spec.domain.cpu.cores
